#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>
#define __USE_GNU
#include <search.h>

#define ALLOC_SIZE 4096

static struct hsearch_data MAJ_MIN_TO_MOUNTP;

typedef struct mount_list {
	char *major_minor;
	struct mount_list *next;
} Mount_list;

Mount_list *mount_list_head = NULL;


/* Make an error for load_file
 * Error will be put into a (char *) in the following format:
 *   "\0error message\0"
 * 
 * So when looking at the message you have to look one character
 * a head. ie.
 * 
 *    printf("Error: %s",error_buf+1);
 * 
 * This is so that we can do the check:
 * 
 * fbuf = load_file("myfile");
 * if (*fbuf == 0)
 *    printf("Error: %s\n",fbuf+1);
 * else
 *    printf("It worked!");
 */
char *
    load_file_error(const char *message, int myerrno)
{
	char *error_buf;       
	
	/* Allocate memory for error */
	if (message == NULL)
	    error_buf = malloc(strlen(strerror(myerrno))+2);	
	else
	    error_buf = malloc(strlen(message)+strlen(strerror(myerrno))+2);

	/* Set first two characters to \0 for strcat */
	bzero(error_buf,2);
	/* Put message in error_buf */
	if (message)
	    strcat((error_buf+1),message);
	
	if (myerrno)
	    strcat((error_buf+1),strerror(myerrno));
	
	return error_buf;
}

/* Load File into memory */

char *
load_file(const char *filename) {
	int res = 1;
	int fptr;
	char *fbuf;
	int myerrno;
	char buffer[ALLOC_SIZE];
	unsigned int tot_size;

	if (!filename) {
		fbuf = load_file_error("filename is NULL!",0);
		return fbuf;
	}
	
	/* Open the filename read only */
	if ((fptr = open(filename,O_RDONLY)) != -1) {
		
		/* initialize fbuf */
		fbuf = malloc(ALLOC_SIZE+1);
		
		/* If malloc failes, report and exit */
		if (!fbuf) {
			fprintf(stderr,"load_file: malloc: %s",strerror(errno));
			exit(1);
		}
		
		*fbuf = 0;
		tot_size = ALLOC_SIZE;

		/* Read file in 4096 chunks */
		while ((res = read(fptr,buffer,ALLOC_SIZE)) > 0) {
			strncat(fbuf,buffer,res);
			if (res < ALLOC_SIZE)
			    break;

			/* realloc more memory if more of the file is expected. */
			fbuf = realloc(fbuf,tot_size+ALLOC_SIZE);
			tot_size += ALLOC_SIZE;

			/* If realloc failes report and exit. */
			if (!fbuf) {
				fprintf(stderr,"load_file: realloc: %s",strerror(errno));
				exit(1);
			}
		}
		
		myerrno = errno;

		close(fptr);

		/* If read fails, put error message into fbuf */
		if (res == -1) {
			free(fbuf);
			fbuf = load_file_error("read: ",myerrno);
		}
	}
	/* If open fails, put error message into fbuf */
	else
	    fbuf = load_file_error("open: ",errno);

	return(fbuf);
}

/* report an error
 * 
 * int STATUS can equal WARNING == 0
 *                      FATAL   == 1
 * 
 * WARNING status will just return while FATAL status will exit the 
 * program with error code 1
 */
void
report_error(int STATUS, int QUIET, const char *format, ...) {
	
	/* Guess we need no more than 100 bytes. */
	int n;
	unsigned int size = ALLOC_SIZE;
	char *str;
	va_list ap;
	
	/* If QUIET is on and STATUS == 1 (fatal) then exit 
	 * else just return */
	if (QUIET && STATUS == 1)
	    exit(1);
	else if (QUIET)
	    return;
	
	if ((str = malloc (size)) == NULL) {
		fprintf(stderr,"report_fatal_error: malloc: %s\n",strerror(errno));
		exit(1);
	}
	
	while (1) {
		/* Try to print in the allocated space. */
		va_start(ap, format);
		n = vsnprintf(str, size, format, ap);
		va_end(ap);

		/* If that worked, print message. */
		if (n > -1 && n < size) {
			if (STATUS == 1)
			    fprintf(stderr,"fatal error: %s",str);
			else if (STATUS == 0)
			    fprintf(stderr,"warning: %s",str);
			else
			    fprintf(stderr,"%s",str);
			fflush(stderr);
			break;
		}

		/* Else try again with more space. */
		if (n > -1)    /* glibc 2.1 */
		    size = n+1; /* precisely what is needed */
		else           /* glibc 2.0 */
		    size *= 2;  /* twice the old size */

		if ((str = realloc (str, size)) == NULL) {
			fprintf(stderr,"report_fatal_error: realloc: %s\n",strerror(errno));
			exit(1);
		}
	}

	if (STATUS == 1)
	    exit(1);
}

/*
 *  * xmalloc() 
 *
 *  allocate n bytes with malloc and warn if there is an error.
 *   */
static void *xmalloc(const unsigned n) {
        void *p;
        p = malloc(n);
        if (!p)
        	fprintf(stderr, "xmalloc: %s\n", strerror(errno));
	return p;
}

/*
 * rlocate_get_maj_min()
 *
 * get major and minor numbers from sysfs
 *
 */
static char * rlocate_get_maj_min(char *dev) {
        FILE *fd;
        char *dev_info_file; 
	int dev_len = strlen(dev);
        int dev_info_file_len = 11; // /sys/block/ 
        char *dev_part;
        char *p, *pp;
        char *major_minor;
        int buffer_len = 8;
        char buffer[buffer_len];
        dev_part = (char *)xmalloc(dev_len + 1);
	if (!dev_part)
		return NULL;
        p = dev;
        pp = dev_part;
        while (!isdigit(*p) && *p !='\0') {
                *pp++ = *p++;
        }
        *pp = '\0';
	dev_info_file_len += dev_len + 4; // hda/hda5/dev
	if (*p != '\0') 
		dev_info_file_len += strlen(dev_part)+1;
        dev_info_file = (char *)xmalloc(dev_info_file_len+1);

	if (!dev_info_file)
		return NULL;
        strcpy(dev_info_file, "/sys/block/");
        if (*p != '\0') {
                strcat(dev_info_file, dev_part);
                strcat(dev_info_file, "/");
        }

        strcat(dev_info_file, dev);
        strcat(dev_info_file, "/dev");
        if ( (fd = fopen(dev_info_file, "r")) != NULL ) {
                fgets(buffer, buffer_len, fd);
                
                if (fclose(fd) <0)
                        fprintf(stderr, "rlocate_get_maj_min(): fclose:"
				" can't close %s: %s\n", 
				dev_info_file, strerror(errno) );
                major_minor = (char *)xmalloc( strlen(buffer)+1 );
		if (!major_minor) 
			return NULL;
                strncpy(major_minor, buffer, strlen(buffer)-1);
		major_minor[strlen(buffer)-1]='\0';
		free(dev_part);
		free(dev_info_file);
                return major_minor;
        }
	free(dev_part);
	free(dev_info_file);
        return NULL;
}

/* 
 * rlocate_create_mount_hash() 
 *
 * create MAJ_MIN_TO_MOUNTP hash.
 */
void rlocate_create_mount_hash() {
        memset( &MAJ_MIN_TO_MOUNTP, 0, sizeof(MAJ_MIN_TO_MOUNTP));
        if (hcreate_r(200, &MAJ_MIN_TO_MOUNTP) == 0) 
                fprintf(stderr, "rlocate_create_mount_hash(): "
				"hcreate_r: cannot create maj_min hash\n");
}

/* 
 * rlocate_update_mount_hash() 
 *
 * update MAJ_MIN_TO_MOUNTP hash.
 */
void rlocate_update_mount_hash() {
        FILE *fd;
        ENTRY mount_entry, *ret_entry;
        int buffer_len = 1000;
        char buffer[buffer_len];
        char *dev;
        char *end_str;
        char *mountpoint, *mp;
        char *major_minor;
        Mount_list *ml;
        /* read mtab and major/minor numbers and mount point */

        if ( (fd = fopen("/etc/mtab", "r")) != NULL ) {
                while ( (fgets(buffer, buffer_len, fd)) != NULL ) {
                        if (!strncmp(buffer, "/dev/", 5)) {

                                /* get dev file like hda1 */
                                end_str = strchr(buffer, ' ');
                                *end_str = '\0';
                                dev = buffer + 5;

                                /* get major/minor numbers */
                                major_minor = rlocate_get_maj_min(dev);

                                /* get mount point */
                                mp = end_str + 2;
                                end_str = strchr(mp, ' ');
                                *end_str = '\0';
                                mountpoint = (char *)xmalloc(strlen(mp) + 1);
				if (!mountpoint)
					return;
                                strcpy(mountpoint, mp);

                                /* put mount point and major/minor in the
                                 * hash */
                                mount_entry.key  = major_minor;
                                if( hsearch_r(mount_entry, FIND, &ret_entry, 
                                              &MAJ_MIN_TO_MOUNTP) == 0) {
                                	mount_entry.data = mountpoint;
                                	if( hsearch_r(mount_entry, ENTER, 
					    	      &ret_entry, 
                                             	      &MAJ_MIN_TO_MOUNTP) == 0) {
                                        	fprintf(stderr, 
						     "hsearch: hash is full\n");
						break;
					} 
                                	/* add mount point to the list */
                                	ml = (Mount_list*)xmalloc(sizeof(Mount_list));
					if (!ml)
						return;
                                	ml->major_minor = major_minor;
                                	if (mount_list_head == NULL)
						ml->next = ml;
					else {
						ml->next=mount_list_head->next;
						mount_list_head->next = ml;
					}
					mount_list_head = ml;
                                } else {
					if ( strcmp(ret_entry->data, 
						    mountpoint) ) {
						free(ret_entry->data);
                                		ret_entry->data = mountpoint;
					} else
						free(mountpoint);
					free(major_minor);
				}
                        }
                }
                if (fclose(fd) <0)
                        fprintf(stderr, "rlocate_update_mount_hash(): fclose:"
				" can't close /etc/mtab: %s\n", 
				strerror(errno) );
        }

}

/* 
 * rlocate_destroy_mount_hash()
 */
void rlocate_destroy_mount_hash() {
	Mount_list *ml, *ml_p;
	char *mountpoint;
        ENTRY mount_entry, *me;
	ml = mount_list_head->next;
	while (1) {
                mount_entry.key = ml->major_minor;
                hsearch_r(mount_entry, FIND, &me, &MAJ_MIN_TO_MOUNTP);
                if ( me ) {
        		mountpoint = (char *)me->data;
			free(mountpoint);
                }
		free(ml->major_minor);
		if (ml == mount_list_head) {
			free(ml);
			break;
		} 
		ml_p = ml;
		ml = ml->next;
		free(ml_p);
	}
        hdestroy_r(&MAJ_MIN_TO_MOUNTP);
}

/* 
 * rlocate_get_mountpoint() 
 *
 * convert directory path with major and minor numBER
 * passed in the buffer string.
 */
void rlocate_get_mountpoint(char *buffer) {
        ENTRY mount_entry, *me;
        char *mountpoint;
        char *dirstr_part;
        char *p;

        p = index(buffer, '/');
	if (!p) {
		p = buffer + strlen(buffer);
	}
	dirstr_part = strdup(p);
	if (!dirstr_part) {
		fprintf(stderr,"rlocate_get_mountpoint: strdup: %s",
							strerror(errno));
		*buffer = '\0';
		return;
	}
	*p = '\0';
        mount_entry.key = buffer;
        hsearch_r(mount_entry, FIND, &me, &MAJ_MIN_TO_MOUNTP);
        if ( !me ) {
		*buffer = '\0';
		free(dirstr_part);
		return;
        }
        mountpoint = (char *)me->data;
	if (!strcmp(mountpoint, "")) 
		strcpy(buffer, dirstr_part + 1);
	else {
        	strcpy(buffer, mountpoint);
        	strcat(buffer, dirstr_part);
	}
        free(dirstr_part);
}
